@@ -23,8 +23,6 @@
 #include <gst/gst.h>
 #include <gconf/gconf-client.h>
 
-#include "gstswitchsink.h"
-
 G_BEGIN_DECLS
 
 #define GST_TYPE_GCONF_VIDEO_SINK \
@@ -34,27 +32,23 @@ G_BEGIN_DECLS
                                GstGConfVideoSink))
 #define GST_GCONF_VIDEO_SINK_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_GCONF_VIDEO_SINK, \
-                            GstGConfVideoSinkClass))
+                            GstGConfVideoSink))
 #define GST_IS_GCONF_VIDEO_SINK(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_GCONF_VIDEO_SINK))
-#define GST_IS_GCONF_VIDEO_SINK_CLASS(klass) \
+#define GST_IS_GCONF_VIDEO_SINK_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_GCONF_VIDEO_SINK))
 
 typedef struct _GstGConfVideoSink {
-  GstSwitchSink parent;
+  GstBin parent;
 
   /* explicit pointers to stuff used */
   GConfClient *client;
-
-  /* gconf notify id */
-  guint notify_id;
-
-  /* Current gconf string */
-  gchar *gconf_str;
+  GstElement *kid;
+  GstPad *pad;
 } GstGConfVideoSink;
 
 typedef struct _GstGConfVideoSinkClass {
-  GstSwitchSinkClass parent_class;
+  GstBinClass parent_class;
 } GstGConfVideoSinkClass;
 
 GType   gst_gconf_video_sink_get_type   (void);
