@@ -15,11 +15,6 @@
  * Boston, MA 02111-1307, USA.
  *
  */
-/**
- * SECTION:element-pngenc
- *
- * Encodes png images.
- */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -32,7 +27,15 @@
 
 #define MAX_HEIGHT              4096
 
-GST_DEBUG_CATEGORY_STATIC (pngenc_debug);
+
+static GstElementDetails gst_pngenc_details = {
+  "PNG encoder",
+  "Codec/Encoder/Image",
+  "Encode a video frame to a .png image",
+  "Jeremy SIMON <jsimon13@yahoo.fr>",
+};
+
+GST_DEBUG_CATEGORY (pngenc_debug);
 #define GST_CAT_DEFAULT pngenc_debug
 
 /* Filter signals and args */
@@ -60,15 +63,14 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("image/png, "
         "width = (int) [ 16, 4096 ], "
-        "height = (int) [ 16, 4096 ], " "framerate = " GST_VIDEO_FPS_RANGE)
+        "height = (int) [ 16, 4096 ], " "framerate = (fraction) [ 0.0, MAX ]")
     );
 
 static GstStaticPadTemplate pngenc_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_RGBA ";" GST_VIDEO_CAPS_RGB ";"
-        GST_VIDEO_CAPS_GRAY8)
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_RGBA ";" GST_VIDEO_CAPS_RGB)
     );
 
 /* static GstElementClass *parent_class = NULL; */
@@ -103,20 +105,19 @@ gst_pngenc_base_init (gpointer g_class)
       (element_class, gst_static_pad_template_get (&pngenc_sink_template));
   gst_element_class_add_pad_template
       (element_class, gst_static_pad_template_get (&pngenc_src_template));
-  gst_element_class_set_details_simple (element_class, "PNG image encoder",
-      "Codec/Encoder/Image",
-      "Encode a video frame to a .png image",
-      "Jeremy SIMON <jsimon13@yahoo.fr>");
+  gst_element_class_set_details (element_class, &gst_pngenc_details);
 }
 
 static void
 gst_pngenc_class_init (GstPngEncClass * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
+  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   gobject_class->get_property = gst_pngenc_get_property;
   gobject_class->set_property = gst_pngenc_set_property;
@@ -124,19 +125,19 @@ gst_pngenc_class_init (GstPngEncClass * klass)
   g_object_class_install_property (gobject_class, ARG_SNAPSHOT,
       g_param_spec_boolean ("snapshot", "Snapshot",
           "Send EOS after encoding a frame, useful for snapshots",
-          DEFAULT_SNAPSHOT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_SNAPSHOT, (GParamFlags) G_PARAM_READWRITE));
 
 /*   g_object_class_install_property (gobject_class, ARG_NEWMEDIA, */
 /*       g_param_spec_boolean ("newmedia", "newmedia", */
 /*           "Send new media discontinuity after encoding each frame", */
-/*           DEFAULT_NEWMEDIA, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)); */
+/*           DEFAULT_NEWMEDIA, (GParamFlags) G_PARAM_READWRITE)); */
 
-  g_object_class_install_property (gobject_class, ARG_COMPRESSION_LEVEL,
+  g_object_class_install_property
+      (gobject_class, ARG_COMPRESSION_LEVEL,
       g_param_spec_uint ("compression-level", "compression-level",
           "PNG compression level",
           Z_NO_COMPRESSION, Z_BEST_COMPRESSION,
-          DEFAULT_COMPRESSION_LEVEL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_COMPRESSION_LEVEL, (GParamFlags) G_PARAM_READWRITE));
 
   GST_DEBUG_CATEGORY_INIT (pngenc_debug, "pngenc", 0, "PNG image encoder");
 }
@@ -150,6 +151,7 @@ gst_pngenc_setcaps (GstPad * pad, GstCaps * caps)
   GstStructure *structure;
   GstCaps *pcaps;
   gboolean ret = TRUE;
+  GstPad *opeer;
 
   pngenc = GST_PNGENC (gst_pad_get_parent (pad));
 
@@ -159,24 +161,21 @@ gst_pngenc_setcaps (GstPad * pad, GstCaps * caps)
   fps = gst_structure_get_value (structure, "framerate");
   gst_structure_get_int (structure, "bpp", &pngenc->bpp);
 
-  if (pngenc->bpp == 32)
-    pngenc->stride = pngenc->width * 4;
-  else if (pngenc->bpp == 8)
-    pngenc->stride = GST_ROUND_UP_4 (pngenc->width);
-  else
-    pngenc->stride = GST_ROUND_UP_4 (pngenc->width * 3);
-
-  pcaps = gst_caps_new_simple ("image/png",
-      "width", G_TYPE_INT, pngenc->width,
-      "height", G_TYPE_INT, pngenc->height, NULL);
-  structure = gst_caps_get_structure (pcaps, 0);
-  gst_structure_set_value (structure, "framerate", fps);
-
-  ret = gst_pad_set_caps (pngenc->srcpad, pcaps);
-
-  gst_caps_unref (pcaps);
-  gst_object_unref (pngenc);
+  opeer = gst_pad_get_peer (pngenc->srcpad);
+  if (opeer) {
+    pcaps = gst_caps_new_simple ("image/png",
+        "width", G_TYPE_INT, pngenc->width,
+        "height", G_TYPE_INT, pngenc->height, NULL);
+    structure = gst_caps_get_structure (pcaps, 0);
+    gst_structure_set_value (structure, "framerate", fps);
 
+    if (gst_pad_accept_caps (opeer, pcaps)) {
+      gst_pad_set_caps (pngenc->srcpad, pcaps);
+    } else
+      ret = FALSE;
+    gst_caps_unref (pcaps);
+    gst_object_unref (opeer);
+  }
   return ret;
 }
 
@@ -184,8 +183,8 @@ static void
 gst_pngenc_init (GstPngEnc * pngenc, GstPngEncClass * g_class)
 {
   /* sinkpad */
-  pngenc->sinkpad = gst_pad_new_from_static_template
-      (&pngenc_sink_template, "sink");
+  pngenc->sinkpad = gst_pad_new_from_template
+      (gst_static_pad_template_get (&pngenc_sink_template), "sink");
   gst_pad_set_chain_function (pngenc->sinkpad, gst_pngenc_chain);
   /*   gst_pad_set_link_function (pngenc->sinkpad, gst_pngenc_sinklink); */
   /*   gst_pad_set_getcaps_function (pngenc->sinkpad, gst_pngenc_sink_getcaps); */
@@ -193,8 +192,8 @@ gst_pngenc_init (GstPngEnc * pngenc, GstPngEncClass * g_class)
   gst_element_add_pad (GST_ELEMENT (pngenc), pngenc->sinkpad);
 
   /* srcpad */
-  pngenc->srcpad = gst_pad_new_from_static_template
-      (&pngenc_src_template, "src");
+  pngenc->srcpad = gst_pad_new_from_template
+      (gst_static_pad_template_get (&pngenc_src_template), "src");
   /*   pngenc->srcpad = gst_pad_new ("src", GST_PAD_SRC); */
   /*   gst_pad_set_getcaps_function (pngenc->srcpad, gst_pngenc_src_getcaps); */
   /*   gst_pad_set_setcaps_function (pngenc->srcpad, gst_pngenc_setcaps); */
@@ -210,27 +209,38 @@ gst_pngenc_init (GstPngEnc * pngenc, GstPngEncClass * g_class)
 }
 
 static void
-user_flush_data (png_structp png_ptr G_GNUC_UNUSED)
-{
-}
-
-static void
-user_write_data (png_structp png_ptr, png_bytep data, png_uint_32 length)
+user_flush_data (png_structp png_ptr)
 {
   GstPngEnc *pngenc;
 
   pngenc = (GstPngEnc *) png_get_io_ptr (png_ptr);
 
-  if (pngenc->written + length >= GST_BUFFER_SIZE (pngenc->buffer_out)) {
-    GST_ERROR_OBJECT (pngenc, "output buffer bigger than the input buffer!?");
-    png_error (png_ptr, "output buffer bigger than the input buffer!?");
+  gst_pad_push_event (pngenc->srcpad, gst_event_new_flush_start ());
+  gst_pad_push_event (pngenc->srcpad, gst_event_new_flush_stop ());
+}
 
-    /* never reached */
-    return;
-  }
 
-  memcpy (GST_BUFFER_DATA (pngenc->buffer_out) + pngenc->written, data, length);
-  pngenc->written += length;
+static void
+user_write_data (png_structp png_ptr, png_bytep data, png_uint_32 length)
+{
+  GstBuffer *buffer;
+  GstPngEnc *pngenc;
+
+  pngenc = (GstPngEnc *) png_get_io_ptr (png_ptr);
+
+  buffer = gst_buffer_new ();
+  GST_BUFFER_DATA (buffer) = g_memdup (data, length);
+  GST_BUFFER_SIZE (buffer) = length;
+
+  if (pngenc->buffer_out) {
+    GstBuffer *merge;
+
+    merge = gst_buffer_merge (pngenc->buffer_out, buffer);
+    gst_buffer_unref (buffer);
+    gst_buffer_unref (pngenc->buffer_out);
+    pngenc->buffer_out = merge;
+  } else
+    pngenc->buffer_out = buffer;
 }
 
 static GstFlowReturn
@@ -238,15 +248,15 @@ gst_pngenc_chain (GstPad * pad, GstBuffer * buf)
 {
   GstPngEnc *pngenc;
   gint row_index;
-  gint color_type;
   png_byte *row_pointers[MAX_HEIGHT];
   GstFlowReturn ret = GST_FLOW_OK;
-  GstBuffer *encoded_buf = NULL;
 
   pngenc = GST_PNGENC (gst_pad_get_parent (pad));
 
   GST_DEBUG_OBJECT (pngenc, "BEGINNING");
 
+  pngenc->buffer_out = NULL;
+
   /* initialize png struct stuff */
   pngenc->png_struct_ptr = png_create_write_struct (PNG_LIBPNG_VER_STRING,
       (png_voidp) NULL, user_error_fn, user_warning_fn);
@@ -269,7 +279,7 @@ gst_pngenc_chain (GstPad * pad, GstBuffer * buf)
   }
 
   /* non-0 return is from a longjmp inside of libpng */
-  if (setjmp (png_jmpbuf (pngenc->png_struct_ptr)) != 0) {
+  if (setjmp (pngenc->png_struct_ptr->jmpbuf) != 0) {
     gst_buffer_unref (buf);
     png_destroy_write_struct (&pngenc->png_struct_ptr, &pngenc->png_info_ptr);
     GST_ELEMENT_ERROR (pngenc, LIBRARY, FAILED, (NULL),
@@ -282,69 +292,58 @@ gst_pngenc_chain (GstPad * pad, GstBuffer * buf)
       PNG_FILTER_NONE | PNG_FILTER_VALUE_NONE);
   png_set_compression_level (pngenc->png_struct_ptr, pngenc->compression_level);
 
-  if (pngenc->bpp == 32)
-    color_type = PNG_COLOR_TYPE_RGBA;
-  else if (pngenc->bpp == 8)
-    color_type = PNG_COLOR_TYPE_GRAY;
-  else
-    color_type = PNG_COLOR_TYPE_RGB;
-
   png_set_IHDR (pngenc->png_struct_ptr,
       pngenc->png_info_ptr,
       pngenc->width,
       pngenc->height,
       8,
-      color_type,
+      (pngenc->bpp == 32) ? PNG_COLOR_TYPE_RGBA : PNG_COLOR_TYPE_RGB,
       PNG_INTERLACE_NONE,
       PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
 
   png_set_write_fn (pngenc->png_struct_ptr, pngenc,
       (png_rw_ptr) user_write_data, user_flush_data);
 
-  for (row_index = 0; row_index < pngenc->height; row_index++) {
+  for (row_index = 0; row_index < pngenc->height; row_index++)
     row_pointers[row_index] = GST_BUFFER_DATA (buf) +
-        (row_index * pngenc->stride);
-  }
-
-  /* allocate the output buffer */
-  pngenc->buffer_out =
-      gst_buffer_new_and_alloc (pngenc->height * pngenc->stride);
-  pngenc->written = 0;
+        (row_index * pngenc->png_info_ptr->rowbytes);
 
   png_write_info (pngenc->png_struct_ptr, pngenc->png_info_ptr);
   png_write_image (pngenc->png_struct_ptr, row_pointers);
   png_write_end (pngenc->png_struct_ptr, NULL);
 
-  encoded_buf = gst_buffer_create_sub (pngenc->buffer_out, 0, pngenc->written);
+  user_flush_data (pngenc->png_struct_ptr);
 
   png_destroy_info_struct (pngenc->png_struct_ptr, &pngenc->png_info_ptr);
   png_destroy_write_struct (&pngenc->png_struct_ptr, (png_infopp) NULL);
-  gst_buffer_copy_metadata (encoded_buf, buf, GST_BUFFER_COPY_TIMESTAMPS);
+  gst_buffer_stamp (pngenc->buffer_out, buf);
   gst_buffer_unref (buf);
-  gst_buffer_set_caps (encoded_buf, GST_PAD_CAPS (pngenc->srcpad));
 
-  if ((ret = gst_pad_push (pngenc->srcpad, encoded_buf)) != GST_FLOW_OK)
+  if ((ret = gst_pad_push (pngenc->srcpad, pngenc->buffer_out)) != GST_FLOW_OK)
     goto done;
 
   if (pngenc->snapshot) {
-    GstEvent *event;
-
     GST_DEBUG_OBJECT (pngenc, "snapshot mode, sending EOS");
     /* send EOS event, since a frame has been pushed out */
-    event = gst_event_new_eos ();
+    GstEvent *event = gst_event_new_eos ();
 
-    gst_pad_push_event (pngenc->srcpad, event);
-    ret = GST_FLOW_UNEXPECTED;
+    ret = gst_pad_push_event (pngenc->srcpad, event);
+
+    if (!(GST_FLOW_IS_FATAL (ret)))
+      ret = GST_FLOW_UNEXPECTED;
   }
+/*  else if (pngenc->newmedia) { */
+/*     /\* send new media discont *\/ */
+/*     GstEvent *newmedia_event; */
 
+/*     newmedia_event = */
+/*         gst_event_new_discontinuous (TRUE, GST_FORMAT_TIME, (gint64) 0, */
+/*         GST_FORMAT_UNDEFINED); */
+/*     ret = gst_pad_push (pngenc->srcpad, GST_DATA (newmedia_event)); */
+/*   } */
 done:
   GST_DEBUG_OBJECT (pngenc, "END, ret:%d", ret);
 
-  if (pngenc->buffer_out != NULL) {
-    gst_buffer_unref (pngenc->buffer_out);
-    pngenc->buffer_out = NULL;
-  }
-
   gst_object_unref (pngenc);
   return ret;
 }
