@@ -23,8 +23,6 @@
 #include "gstspeexdec.h"
 #include "gstspeexenc.h"
 
-#include <gst/tag/tag.h>
-
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
@@ -37,8 +35,6 @@ plugin_init (GstPlugin * plugin)
           GST_TYPE_SPEEXDEC))
     return FALSE;
 
-  gst_tag_register_musicbrainz_tags ();
-
   return TRUE;
 }
 
