@@ -1,8 +1,5 @@
 /* GStreamer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
- * Copyright (C) <2007> Wim Taymans <wim.taymans@collabora.co.uk>
- * Copyright (C) <2007> Edward Hervey <edward.hervey@collabora.co.uk>
- * Copyright (C) <2007> Jan Schmidt <thaytan@noraisin.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,18 +21,11 @@
 #include "config.h"
 #endif
 #include <gst/gst.h>
-#include <gst/base/gstbasetransform.h>
 #include <gst/video/video.h>
-#include <gst/controller/gstcontroller.h>
 
-#include <stdlib.h>
 #include <string.h>
 #include <math.h>
 
-#ifndef M_PI
-#define M_PI  3.14159265358979323846
-#endif
-
 #define GST_TYPE_ALPHA \
   (gst_alpha_get_type())
 #define GST_ALPHA(obj) \
@@ -44,7 +34,7 @@
   (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_ALPHA,GstAlphaClass))
 #define GST_IS_ALPHA(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ALPHA))
-#define GST_IS_ALPHA_CLASS(klass) \
+#define GST_IS_ALPHA_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_ALPHA))
 
 typedef struct _GstAlpha GstAlpha;
@@ -59,16 +49,21 @@ typedef enum
 }
 GstAlphaMethod;
 
-GST_DEBUG_CATEGORY_STATIC (gst_alpha_debug);
-#define GST_CAT_DEFAULT gst_alpha_debug
+#define ROUND_UP_2(x) (((x) + 1) & ~1)
+#define ROUND_UP_4(x) (((x) + 3) & ~3)
+#define ROUND_UP_8(x) (((x) + 7) & ~7)
 
 struct _GstAlpha
 {
-  GstBaseTransform parent;
+  GstElement element;
+
+  /* pads */
+  GstPad *sinkpad;
+  GstPad *srcpad;
 
   /* caps */
-  GstVideoFormat format;
-  gint width, height;
+  gint in_width, in_height;
+  gint out_width, out_height;
   gboolean ayuv;
 
   gdouble alpha;
@@ -81,8 +76,6 @@ struct _GstAlpha
 
   gfloat angle;
   gfloat noise_level;
-  guint black_sensitivity;
-  guint white_sensitivity;
 
   gfloat y;                     /* chroma color */
   gint8 cb, cr;
@@ -97,17 +90,16 @@ struct _GstAlpha
 
 struct _GstAlphaClass
 {
-  GstBaseTransformClass parent_class;
+  GstElementClass parent_class;
 };
 
 /* elementfactory information */
-static const GstElementDetails gst_alpha_details =
-GST_ELEMENT_DETAILS ("Alpha filter",
+static GstElementDetails gst_alpha_details =
+GST_ELEMENT_DETAILS ("alpha filter",
     "Filter/Effect/Video",
-    "Adds an alpha channel to video - uniform or via chroma-keying",
-    "Wim Taymans <wim@fluendo.com>\n"
-    "Edward Hervey <edward.hervey@collabora.co.uk>\n"
-    "Jan Schmidt <thaytan@noraisin.net>");
+    "Adds an alpha channel to video",
+    "Wim Taymans <wim@fluendo.com>");
+
 
 /* Alpha signals and args */
 enum
@@ -123,22 +115,18 @@ enum
 #define DEFAULT_TARGET_B 0
 #define DEFAULT_ANGLE 20.0
 #define DEFAULT_NOISE_LEVEL 2.0
-#define DEFAULT_BLACK_SENSITIVITY 100
-#define DEFAULT_WHITE_SENSITIVITY 100
 
 enum
 {
-  PROP_0,
-  PROP_METHOD,
-  PROP_ALPHA,
-  PROP_TARGET_R,
-  PROP_TARGET_G,
-  PROP_TARGET_B,
-  PROP_ANGLE,
-  PROP_NOISE_LEVEL,
-  PROP_BLACK_SENSITIVITY,
-  PROP_WHITE_SENSITIVITY,
-  PROP_LAST
+  ARG_0,
+  ARG_METHOD,
+  ARG_ALPHA,
+  ARG_TARGET_R,
+  ARG_TARGET_G,
+  ARG_TARGET_B,
+  ARG_ANGLE,
+  ARG_NOISE_LEVEL,
+  /* FILL ME */
 };
 
 static GstStaticPadTemplate gst_alpha_src_template =
@@ -152,20 +140,15 @@ static GstStaticPadTemplate gst_alpha_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("AYUV") ";" GST_VIDEO_CAPS_YUV ("I420")
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("AYUV")
+        ";" GST_VIDEO_CAPS_YUV ("I420")
     )
     );
 
-static gboolean gst_alpha_start (GstBaseTransform * trans);
-static gboolean gst_alpha_get_unit_size (GstBaseTransform * btrans,
-    GstCaps * caps, guint * size);
-static GstCaps *gst_alpha_transform_caps (GstBaseTransform * btrans,
-    GstPadDirection direction, GstCaps * caps);
-static gboolean gst_alpha_set_caps (GstBaseTransform * btrans,
-    GstCaps * incaps, GstCaps * outcaps);
-static GstFlowReturn gst_alpha_transform (GstBaseTransform * btrans,
-    GstBuffer * in, GstBuffer * out);
 
+static void gst_alpha_base_init (gpointer g_class);
+static void gst_alpha_class_init (GstAlphaClass * klass);
+static void gst_alpha_init (GstAlpha * alpha);
 static void gst_alpha_init_params (GstAlpha * alpha);
 
 static void gst_alpha_set_property (GObject * object, guint prop_id,
@@ -173,15 +156,21 @@ static void gst_alpha_set_property (GObject * object, guint prop_id,
 static void gst_alpha_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-GST_BOILERPLATE (GstAlpha, gst_alpha, GstBaseTransform,
-    GST_TYPE_BASE_TRANSFORM);
+static gboolean gst_alpha_sink_setcaps (GstPad * pad, GstCaps * caps);
+static GstFlowReturn gst_alpha_chain (GstPad * pad, GstBuffer * buffer);
+
+static GstStateChangeReturn gst_alpha_change_state (GstElement * element,
+    GstStateChange transition);
+
+
+static GstElementClass *parent_class = NULL;
 
 #define GST_TYPE_ALPHA_METHOD (gst_alpha_method_get_type())
 static GType
 gst_alpha_method_get_type (void)
 {
   static GType alpha_method_type = 0;
-  static const GEnumValue alpha_method[] = {
+  static GEnumValue alpha_method[] = {
     {ALPHA_METHOD_SET, "Set/adjust alpha channel", "set"},
     {ALPHA_METHOD_GREEN, "Chroma Key green", "green"},
     {ALPHA_METHOD_BLUE, "Chroma Key blue", "blue"},
@@ -195,6 +184,32 @@ gst_alpha_method_get_type (void)
   return alpha_method_type;
 }
 
+/* static guint gst_alpha_signals[LAST_SIGNAL] = { 0 }; */
+
+GType
+gst_alpha_get_type (void)
+{
+  static GType alpha_type = 0;
+
+  if (!alpha_type) {
+    static const GTypeInfo alpha_info = {
+      sizeof (GstAlphaClass),
+      gst_alpha_base_init,
+      NULL,
+      (GClassInitFunc) gst_alpha_class_init,
+      NULL,
+      NULL,
+      sizeof (GstAlpha),
+      0,
+      (GInstanceInitFunc) gst_alpha_init,
+    };
+
+    alpha_type =
+        g_type_register_static (GST_TYPE_ELEMENT, "GstAlpha", &alpha_info, 0);
+  }
+  return alpha_type;
+}
+
 static void
 gst_alpha_base_init (gpointer g_class)
 {
@@ -206,72 +221,63 @@ gst_alpha_base_init (gpointer g_class)
       gst_static_pad_template_get (&gst_alpha_sink_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_alpha_src_template));
-
-  GST_DEBUG_CATEGORY_INIT (gst_alpha_debug, "alpha", 0,
-      "alpha - Element for adding alpha channel to streams");
 }
 static void
 gst_alpha_class_init (GstAlphaClass * klass)
 {
   GObjectClass *gobject_class;
-  GstBaseTransformClass *btrans_class;
+  GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
-  btrans_class = (GstBaseTransformClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
+
+  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   gobject_class->set_property = gst_alpha_set_property;
   gobject_class->get_property = gst_alpha_get_property;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_METHOD,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_METHOD,
       g_param_spec_enum ("method", "Method",
           "How the alpha channels should be created", GST_TYPE_ALPHA_METHOD,
           DEFAULT_METHOD, (GParamFlags) G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_ALPHA,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_ALPHA,
       g_param_spec_double ("alpha", "Alpha", "The value for the alpha channel",
-          0.0, 1.0, DEFAULT_ALPHA,
-          (GParamFlags) G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_TARGET_R,
-      g_param_spec_uint ("target_r", "Target Red", "The Red target", 0, 255,
-          DEFAULT_TARGET_R,
-          (GParamFlags) G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_TARGET_G,
-      g_param_spec_uint ("target_g", "Target Green", "The Green target", 0, 255,
-          DEFAULT_TARGET_G,
-          (GParamFlags) G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_TARGET_B,
-      g_param_spec_uint ("target_b", "Target Blue", "The Blue target", 0, 255,
-          DEFAULT_TARGET_B,
-          (GParamFlags) G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_ANGLE,
+          0.0, 1.0, DEFAULT_ALPHA, (GParamFlags) G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_TARGET_R,
+      g_param_spec_uint ("target_r", "Target Red", "The Red target", 0,
+          255, DEFAULT_TARGET_R, (GParamFlags) G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_TARGET_G,
+      g_param_spec_uint ("target_g", "Target Green", "The Green target", 0,
+          255, DEFAULT_TARGET_G, (GParamFlags) G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_TARGET_B,
+      g_param_spec_uint ("target_b", "Target Blue", "The Blue target",
+          0, 255, DEFAULT_TARGET_B, (GParamFlags) G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_ANGLE,
       g_param_spec_float ("angle", "Angle", "Size of the colorcube to change",
-          0.0, 90.0, DEFAULT_ANGLE,
-          (GParamFlags) G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_NOISE_LEVEL,
+          0.0, 90.0, DEFAULT_ANGLE, (GParamFlags) G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NOISE_LEVEL,
       g_param_spec_float ("noise_level", "Noise Level", "Size of noise radius",
-          0.0, 64.0, DEFAULT_NOISE_LEVEL,
-          (GParamFlags) G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass),
-      PROP_BLACK_SENSITIVITY, g_param_spec_uint ("black-sensitivity",
-          "Black Sensitivity", "Sensitivity to dark colors", 0, 128,
-          DEFAULT_BLACK_SENSITIVITY,
-          (GParamFlags) G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass),
-      PROP_WHITE_SENSITIVITY, g_param_spec_uint ("white-sensitivity",
-          "Sensitivity", "Sensitivity to bright colors", 0, 128,
-          DEFAULT_WHITE_SENSITIVITY,
-          (GParamFlags) G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
+          0.0, 64.0, DEFAULT_NOISE_LEVEL, (GParamFlags) G_PARAM_READWRITE));
 
-
-  btrans_class->start = GST_DEBUG_FUNCPTR (gst_alpha_start);
-  btrans_class->transform = GST_DEBUG_FUNCPTR (gst_alpha_transform);
-  btrans_class->get_unit_size = GST_DEBUG_FUNCPTR (gst_alpha_get_unit_size);
-  btrans_class->transform_caps = GST_DEBUG_FUNCPTR (gst_alpha_transform_caps);
-  btrans_class->set_caps = GST_DEBUG_FUNCPTR (gst_alpha_set_caps);
+  gstelement_class->change_state = gst_alpha_change_state;
 }
 
 static void
-gst_alpha_init (GstAlpha * alpha, GstAlphaClass * klass)
+gst_alpha_init (GstAlpha * alpha)
 {
+  /* create the sink and src pads */
+  alpha->sinkpad =
+      gst_pad_new_from_template (gst_static_pad_template_get
+      (&gst_alpha_sink_template), "sink");
+  gst_element_add_pad (GST_ELEMENT (alpha), alpha->sinkpad);
+  gst_pad_set_chain_function (alpha->sinkpad, gst_alpha_chain);
+  gst_pad_set_setcaps_function (alpha->sinkpad, gst_alpha_sink_setcaps);
+
+  alpha->srcpad =
+      gst_pad_new_from_template (gst_static_pad_template_get
+      (&gst_alpha_src_template), "src");
+  gst_element_add_pad (GST_ELEMENT (alpha), alpha->srcpad);
+
   alpha->alpha = DEFAULT_ALPHA;
   alpha->method = DEFAULT_METHOD;
   alpha->target_r = DEFAULT_TARGET_R;
@@ -279,8 +285,6 @@ gst_alpha_init (GstAlpha * alpha, GstAlphaClass * klass)
   alpha->target_b = DEFAULT_TARGET_B;
   alpha->angle = DEFAULT_ANGLE;
   alpha->noise_level = DEFAULT_NOISE_LEVEL;
-  alpha->black_sensitivity = DEFAULT_BLACK_SENSITIVITY;
-  alpha->white_sensitivity = DEFAULT_WHITE_SENSITIVITY;
 }
 
 /* do we need this function? */
@@ -295,7 +299,7 @@ gst_alpha_set_property (GObject * object, guint prop_id,
   alpha = GST_ALPHA (object);
 
   switch (prop_id) {
-    case PROP_METHOD:
+    case ARG_METHOD:
       alpha->method = g_value_get_enum (value);
       switch (alpha->method) {
         case ALPHA_METHOD_GREEN:
@@ -313,35 +317,29 @@ gst_alpha_set_property (GObject * object, guint prop_id,
       }
       gst_alpha_init_params (alpha);
       break;
-    case PROP_ALPHA:
+    case ARG_ALPHA:
       alpha->alpha = g_value_get_double (value);
       break;
-    case PROP_TARGET_R:
+    case ARG_TARGET_R:
       alpha->target_r = g_value_get_uint (value);
       gst_alpha_init_params (alpha);
       break;
-    case PROP_TARGET_G:
+    case ARG_TARGET_G:
       alpha->target_g = g_value_get_uint (value);
       gst_alpha_init_params (alpha);
       break;
-    case PROP_TARGET_B:
+    case ARG_TARGET_B:
       alpha->target_b = g_value_get_uint (value);
       gst_alpha_init_params (alpha);
       break;
-    case PROP_ANGLE:
+    case ARG_ANGLE:
       alpha->angle = g_value_get_float (value);
       gst_alpha_init_params (alpha);
       break;
-    case PROP_NOISE_LEVEL:
+    case ARG_NOISE_LEVEL:
       alpha->noise_level = g_value_get_float (value);
       gst_alpha_init_params (alpha);
       break;
-    case PROP_BLACK_SENSITIVITY:
-      alpha->black_sensitivity = g_value_get_uint (value);
-      break;
-    case PROP_WHITE_SENSITIVITY:
-      alpha->white_sensitivity = g_value_get_uint (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -358,33 +356,27 @@ gst_alpha_get_property (GObject * object, guint prop_id, GValue * value,
   alpha = GST_ALPHA (object);
 
   switch (prop_id) {
-    case PROP_METHOD:
+    case ARG_METHOD:
       g_value_set_enum (value, alpha->method);
       break;
-    case PROP_ALPHA:
+    case ARG_ALPHA:
       g_value_set_double (value, alpha->alpha);
       break;
-    case PROP_TARGET_R:
+    case ARG_TARGET_R:
       g_value_set_uint (value, alpha->target_r);
       break;
-    case PROP_TARGET_G:
+    case ARG_TARGET_G:
       g_value_set_uint (value, alpha->target_g);
       break;
-    case PROP_TARGET_B:
+    case ARG_TARGET_B:
       g_value_set_uint (value, alpha->target_b);
       break;
-    case PROP_ANGLE:
+    case ARG_ANGLE:
       g_value_set_float (value, alpha->angle);
       break;
-    case PROP_NOISE_LEVEL:
+    case ARG_NOISE_LEVEL:
       g_value_set_float (value, alpha->noise_level);
       break;
-    case PROP_BLACK_SENSITIVITY:
-      g_value_set_uint (value, alpha->black_sensitivity);
-      break;
-    case PROP_WHITE_SENSITIVITY:
-      g_value_set_uint (value, alpha->white_sensitivity);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -392,76 +384,33 @@ gst_alpha_get_property (GObject * object, guint prop_id, GValue * value,
 }
 
 static gboolean
-gst_alpha_get_unit_size (GstBaseTransform * btrans,
-    GstCaps * caps, guint * size)
+gst_alpha_sink_setcaps (GstPad * pad, GstCaps * caps)
 {
-  GstVideoFormat format;
-  gint width, height;
-
-  if (!gst_video_format_parse_caps (caps, &format, &width, &height))
-    return FALSE;
-
-  *size = gst_video_format_get_size (format, width, height);
-
-  GST_DEBUG_OBJECT (btrans, "unit size = %d for format %d w %d height %d",
-      *size, format, width, height);
-
-  return TRUE;
-}
-
-static GstCaps *
-gst_alpha_transform_caps (GstBaseTransform * btrans,
-    GstPadDirection direction, GstCaps * caps)
-{
-  GstCaps *ret;
+  GstAlpha *alpha;
   GstStructure *structure;
-  gint i;
+  gboolean ret;
+  guint32 fourcc;
 
-  ret = gst_caps_copy (caps);
+  alpha = GST_ALPHA (GST_PAD_PARENT (pad));
+  structure = gst_caps_get_structure (caps, 0);
 
-  /* When going from the SINK pad to the src, we just need to make sure the
-   * format is AYUV */
-  if (direction == GST_PAD_SINK) {
-    for (i = 0; i < gst_caps_get_size (ret); i++) {
-      structure = gst_caps_get_structure (ret, i);
-      gst_structure_set (structure, "format",
-          GST_TYPE_FOURCC, GST_MAKE_FOURCC ('A', 'Y', 'U', 'V'), NULL);
+  if (gst_structure_get_fourcc (structure, "format", &fourcc)) {
+    switch (fourcc) {
+      case GST_MAKE_FOURCC ('I', '4', '2', '0'):
+        alpha->ayuv = FALSE;
+        break;
+      case GST_MAKE_FOURCC ('A', 'Y', 'U', 'V'):
+        alpha->ayuv = TRUE;
+        break;
+      default:
+        return FALSE;
     }
   } else {
-    GstCaps *ayuv_caps;
-
-    /* In the other direction, prepend a copy of the caps with format AYUV, 
-     * and set the first to I420 */
-    ayuv_caps = gst_caps_copy (ret);
-
-    for (i = 0; i < gst_caps_get_size (ret); i++) {
-      structure = gst_caps_get_structure (ret, i);
-      gst_structure_set (structure, "format",
-          GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I', '4', '2', '0'), NULL);
-    }
-
-    gst_caps_append (ret, ayuv_caps);
+    return FALSE;
   }
 
-  gst_caps_do_simplify (ret);
-
-  return ret;
-}
-
-static gboolean
-gst_alpha_set_caps (GstBaseTransform * btrans,
-    GstCaps * incaps, GstCaps * outcaps)
-{
-  GstAlpha *alpha = GST_ALPHA (btrans);
-
-  if (!gst_video_format_parse_caps (incaps, &alpha->format,
-          &alpha->width, &alpha->height))
-    return FALSE;
-
-  if (alpha->format == GST_VIDEO_FORMAT_AYUV)
-    alpha->ayuv = TRUE;
-  else
-    alpha->ayuv = FALSE;
+  ret = gst_structure_get_int (structure, "width", &alpha->in_width);
+  ret &= gst_structure_get_int (structure, "height", &alpha->in_height);
 
   return TRUE;
 }
@@ -471,20 +420,28 @@ gst_alpha_set_ayuv (guint8 * src, guint8 * dest, gint width, gint height,
     gdouble alpha)
 {
   gint b_alpha = (gint) (alpha * 255);
-  gint y, x;
+  gint i, j;
   gint size;
   gint stride;
+  gint wrap;
 
-  stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_AYUV, 0, width);
-  size = gst_video_format_get_size (GST_VIDEO_FORMAT_AYUV, width, height);
+  width = ROUND_UP_2 (width);
+  height = ROUND_UP_2 (height);
 
-  for (y = 0; y < height; y++) {
-    for (x = 0; x < width; x++) {
+  stride = ROUND_UP_4 (width);
+  size = stride * height;
+
+  wrap = stride - width;
+
+  for (i = 0; i < height; i++) {
+    for (j = 0; j < width; j++) {
       *dest++ = (*src++ * b_alpha) >> 8;
       *dest++ = *src++;
       *dest++ = *src++;
       *dest++ = *src++;
     }
+    src += wrap;
+    dest += wrap;
   }
 }
 
@@ -497,23 +454,24 @@ gst_alpha_set_i420 (guint8 * src, guint8 * dest, gint width, gint height,
   guint8 *srcU;
   guint8 *srcV;
   gint i, j;
-  gint src_wrap, src_uv_wrap;
-  gint y_stride, uv_stride;
-  gboolean odd_width;
+  gint size, size2;
+  gint stride, stride2;
+  gint wrap, wrap2;
 
-  y_stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 0, width);
-  uv_stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 1, width);
+  width = ROUND_UP_2 (width);
+  height = ROUND_UP_2 (height);
 
-  src_wrap = y_stride - width;
-  src_uv_wrap = uv_stride - (width / 2);
+  stride = ROUND_UP_4 (width);
+  size = stride * height;
+  stride2 = ROUND_UP_8 (width) / 2;
+  size2 = stride2 * height / 2;
+
+  wrap = stride - 2 * (width / 2);
+  wrap2 = stride2 - width / 2;
 
   srcY = src;
-  srcU = src + gst_video_format_get_component_offset (GST_VIDEO_FORMAT_I420,
-      1, width, height);
-  srcV = src + gst_video_format_get_component_offset (GST_VIDEO_FORMAT_I420,
-      2, width, height);
-
-  odd_width = (width % 2 != 0);
+  srcU = srcY + size;
+  srcV = srcU + size2;
 
   for (i = 0; i < height; i++) {
     for (j = 0; j < width / 2; j++) {
@@ -526,21 +484,14 @@ gst_alpha_set_i420 (guint8 * src, guint8 * dest, gint width, gint height,
       *dest++ = *srcU++;
       *dest++ = *srcV++;
     }
-    /* Might have one odd column left to do */
-    if (odd_width) {
-      *dest++ = b_alpha;
-      *dest++ = *srcY++;
-      *dest++ = *srcU;
-      *dest++ = *srcV;
-    }
     if (i % 2 == 0) {
       srcU -= width / 2;
       srcV -= width / 2;
     } else {
-      srcU += src_uv_wrap;
-      srcV += src_uv_wrap;
+      srcU += wrap2;
+      srcV += wrap2;
     }
-    srcY += src_wrap;
+    srcY += wrap;
   }
 }
 
@@ -553,16 +504,23 @@ gst_alpha_chroma_key_ayuv (guint8 * src, guint8 * dest, gint width, gint height,
   guint8 *dest1;
   gint i, j;
   gint x, z, u, v, y, a;
+  gint size;
+  gint stride;
+  gint wrap;
   gint tmp, tmp1;
   gint x1, y1;
-  gint smin, smax;
 
-  smin = 128 - alpha->black_sensitivity;
-  smax = 128 + alpha->white_sensitivity;
+  width = ROUND_UP_2 (width);
+  height = ROUND_UP_2 (height);
+
+  stride = ROUND_UP_4 (width);
+  size = stride * height;
 
   src1 = src;
   dest1 = dest;
 
+  wrap = stride - width;
+
   for (i = 0; i < height; i++) {
     for (j = 0; j < width; j++) {
       a = *src1++ * (alpha->alpha);
@@ -570,71 +528,66 @@ gst_alpha_chroma_key_ayuv (guint8 * src, guint8 * dest, gint width, gint height,
       u = *src1++ - 128;
       v = *src1++ - 128;
 
-      if (y < smin || y > smax) {
-        /* too dark or too bright, keep alpha */
+      /* Convert foreground to XZ coords where X direction is defined by
+         the key color */
+      tmp = ((short) u * alpha->cb + (short) v * alpha->cr) >> 7;
+      x = CLAMP (tmp, -128, 127);
+      tmp = ((short) v * alpha->cb - (short) u * alpha->cr) >> 7;
+      z = CLAMP (tmp, -128, 127);
+
+      /* WARNING: accept angle should never be set greater than "somewhat less
+         than 90 degrees" to avoid dealing with negative/infinite tg. In reality,
+         80 degrees should be enough if foreground is reasonable. If this seems
+         to be a problem, go to alternative ways of checking point position
+         (scalar product or line equations). This angle should not be too small
+         either to avoid infinite ctg (used to suppress foreground without use of
+         division) */
+
+      tmp = ((short) (x) * alpha->accept_angle_tg) >> 4;
+      tmp = MIN (tmp, 127);
+
+      if (abs (z) > tmp) {
+        /* keep foreground Kfg = 0 */
         b_alpha = a;
       } else {
-        /* Convert foreground to XZ coords where X direction is defined by
-           the key color */
-        tmp = ((short) u * alpha->cb + (short) v * alpha->cr) >> 7;
-        x = CLAMP (tmp, -128, 127);
-        tmp = ((short) v * alpha->cb - (short) u * alpha->cr) >> 7;
-        z = CLAMP (tmp, -128, 127);
+        /* Compute Kfg (implicitly) and Kbg, suppress foreground in XZ coord
+           according to Kfg */
+        tmp = ((short) (z) * alpha->accept_angle_ctg) >> 4;
+        tmp = CLAMP (tmp, -128, 127);
+        x1 = abs (tmp);
+        y1 = z;
 
-        /* WARNING: accept angle should never be set greater than "somewhat less
-           than 90 degrees" to avoid dealing with negative/infinite tg. In reality,
-           80 degrees should be enough if foreground is reasonable. If this seems
-           to be a problem, go to alternative ways of checking point position
-           (scalar product or line equations). This angle should not be too small
-           either to avoid infinite ctg (used to suppress foreground without use of
-           division) */
+        tmp1 = x - x1;
+        tmp1 = MAX (tmp1, 0);
+        b_alpha = (((unsigned char) (tmp1) *
+                (unsigned short) (alpha->one_over_kc)) / 2);
+        b_alpha = 255 - CLAMP (b_alpha, 0, 255);
+        b_alpha = (a * b_alpha) >> 8;
 
-        tmp = ((short) (x) * alpha->accept_angle_tg) >> 4;
-        tmp = MIN (tmp, 127);
+        tmp = ((unsigned short) (tmp1) * alpha->kfgy_scale) >> 4;
+        tmp1 = MIN (tmp, 255);
 
-        if (abs (z) > tmp) {
-          /* keep foreground Kfg = 0 */
-          b_alpha = a;
-        } else {
-          /* Compute Kfg (implicitly) and Kbg, suppress foreground in XZ coord
-             according to Kfg */
-          tmp = ((short) (z) * alpha->accept_angle_ctg) >> 4;
-          tmp = CLAMP (tmp, -128, 127);
-          x1 = abs (tmp);
-          y1 = z;
+        tmp = y - tmp1;
+        y = MAX (tmp, 0);
 
-          tmp1 = x - x1;
-          tmp1 = MAX (tmp1, 0);
-          b_alpha = (((unsigned char) (tmp1) *
-                  (unsigned short) (alpha->one_over_kc)) / 2);
-          b_alpha = 255 - CLAMP (b_alpha, 0, 255);
-          b_alpha = (a * b_alpha) >> 8;
+        /* Convert suppressed foreground back to CbCr */
+        tmp = ((char) (x1) * (short) (alpha->cb) -
+            (char) (y1) * (short) (alpha->cr)) >> 7;
+        u = CLAMP (tmp, -128, 127);
 
-          tmp = ((unsigned short) (tmp1) * alpha->kfgy_scale) >> 4;
-          tmp1 = MIN (tmp, 255);
+        tmp = ((char) (x1) * (short) (alpha->cr) +
+            (char) (y1) * (short) (alpha->cb)) >> 7;
+        v = CLAMP (tmp, -128, 127);
 
-          tmp = y - tmp1;
-          y = MAX (tmp, 0);
+        /* Deal with noise. For now, a circle around the key color with
+           radius of noise_level treated as exact key color. Introduces
+           sharp transitions.
+         */
+        tmp = z * (short) (z) + (x - alpha->kg) * (short) (x - alpha->kg);
+        tmp = MIN (tmp, 0xffff);
 
-          /* Convert suppressed foreground back to CbCr */
-          tmp = ((char) (x1) * (short) (alpha->cb) -
-              (char) (y1) * (short) (alpha->cr)) >> 7;
-          u = CLAMP (tmp, -128, 127);
-
-          tmp = ((char) (x1) * (short) (alpha->cr) +
-              (char) (y1) * (short) (alpha->cb)) >> 7;
-          v = CLAMP (tmp, -128, 127);
-
-          /* Deal with noise. For now, a circle around the key color with
-             radius of noise_level treated as exact key color. Introduces
-             sharp transitions.
-           */
-          tmp = z * (short) (z) + (x - alpha->kg) * (short) (x - alpha->kg);
-          tmp = MIN (tmp, 0xffff);
-
-          if (tmp < alpha->noise_level * alpha->noise_level) {
-            b_alpha = 0;
-          }
+        if (tmp < alpha->noise_level * alpha->noise_level) {
+          b_alpha = 0;
         }
       }
 
@@ -646,38 +599,59 @@ gst_alpha_chroma_key_ayuv (guint8 * src, guint8 * dest, gint width, gint height,
       *dest1++ = u;
       *dest1++ = v;
     }
+    dest1 += wrap;
+    src1 += wrap;
   }
 }
 
+/* based on http://www.cs.utah.edu/~michael/chroma/
+ */
 static void
-gst_alpha_chromakey_row_i420 (GstAlpha * alpha, guint8 * dest1, guint8 * dest2,
-    guint8 * srcY1, guint8 * srcY2, guint8 * srcU, guint8 * srcV, gint width)
+gst_alpha_chroma_key_i420 (guint8 * src, guint8 * dest, gint width, gint height,
+    GstAlpha * alpha)
 {
-  gint xpos;
   gint b_alpha;
+  guint8 *srcY1, *srcY2, *srcU, *srcV;
+  guint8 *dest1, *dest2;
+  gint i, j;
   gint x, z, u, v, y11, y12, y21, y22, a;
+  gint size, size2;
+  gint stride, stride2;
+  gint wrap, wrap2, wrap3;
   gint tmp, tmp1;
   gint x1, y1;
-  gint smin, smax;
+
+  width = ROUND_UP_2 (width);
+  height = ROUND_UP_2 (height);
+
+  stride = ROUND_UP_4 (width);
+  size = stride * height;
+  stride2 = ROUND_UP_8 (width) / 2;
+  size2 = stride2 * height / 2;
+
+  srcY1 = src;
+  srcY2 = src + stride;
+  srcU = srcY1 + size;
+  srcV = srcU + size2;
+
+  dest1 = dest;
+  dest2 = dest + width * 4;
+
+  wrap = 2 * stride - 2 * (width / 2);
+  wrap2 = stride2 - width / 2;
+  wrap3 = 8 * width - 8 * (width / 2);
 
   a = 255 * alpha->alpha;
-  smin = 128 - alpha->black_sensitivity;
-  smax = 128 + alpha->white_sensitivity;
 
-  for (xpos = 0; xpos < width / 2; xpos++) {
-    y11 = *srcY1++;
-    y12 = *srcY1++;
-    y21 = *srcY2++;
-    y22 = *srcY2++;
-    u = *srcU++ - 128;
-    v = *srcV++ - 128;
+  for (i = 0; i < height / 2; i++) {
+    for (j = 0; j < width / 2; j++) {
+      y11 = *srcY1++;
+      y12 = *srcY1++;
+      y21 = *srcY2++;
+      y22 = *srcY2++;
+      u = *srcU++ - 128;
+      v = *srcV++ - 128;
 
-    if (y11 < smin || y11 > smax ||
-        y12 < smin || y12 > smax ||
-        y21 < smin || y21 > smax || y22 < smin || y22 > smax) {
-      /* too dark or too bright, make opaque */
-      b_alpha = 255;
-    } else {
       /* Convert foreground to XZ coords where X direction is defined by
          the key color */
       tmp = ((short) u * alpha->cb + (short) v * alpha->cr) >> 7;
@@ -747,75 +721,33 @@ gst_alpha_chromakey_row_i420 (GstAlpha * alpha, guint8 * dest1, guint8 * dest2,
           b_alpha = 0;
         }
       }
+
+      u += 128;
+      v += 128;
+
+      *dest1++ = b_alpha;
+      *dest1++ = y11;
+      *dest1++ = u;
+      *dest1++ = v;
+      *dest1++ = b_alpha;
+      *dest1++ = y12;
+      *dest1++ = u;
+      *dest1++ = v;
+      *dest2++ = b_alpha;
+      *dest2++ = y21;
+      *dest2++ = u;
+      *dest2++ = v;
+      *dest2++ = b_alpha;
+      *dest2++ = y22;
+      *dest2++ = u;
+      *dest2++ = v;
     }
-
-    u += 128;
-    v += 128;
-
-    *dest1++ = b_alpha;
-    *dest1++ = y11;
-    *dest1++ = u;
-    *dest1++ = v;
-    *dest1++ = b_alpha;
-    *dest1++ = y12;
-    *dest1++ = u;
-    *dest1++ = v;
-
-    *dest2++ = b_alpha;
-    *dest2++ = y21;
-    *dest2++ = u;
-    *dest2++ = v;
-    *dest2++ = b_alpha;
-    *dest2++ = y22;
-    *dest2++ = u;
-    *dest2++ = v;
-  }
-}
-
-/* based on http://www.cs.utah.edu/~michael/chroma/
- */
-static void
-gst_alpha_chroma_key_i420 (guint8 * src, guint8 * dest, gint width, gint height,
-    GstAlpha * alpha)
-{
-  guint8 *srcY1, *srcY2, *srcU, *srcV;
-  guint8 *dest1, *dest2;
-  gint ypos;
-  gint dest_stride, src_y_stride, src_uv_stride;
-
-  dest_stride =
-      gst_video_format_get_row_stride (GST_VIDEO_FORMAT_AYUV, 0, width);
-  src_y_stride =
-      gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 0, width);
-  src_uv_stride =
-      gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 1, width);
-
-  srcY1 = src;
-  srcY2 = src + src_y_stride;
-
-  srcU = src + gst_video_format_get_component_offset (GST_VIDEO_FORMAT_I420,
-      1, width, height);
-  srcV = src + gst_video_format_get_component_offset (GST_VIDEO_FORMAT_I420,
-      2, width, height);
-
-  dest1 = dest;
-  dest2 = dest + dest_stride;
-
-  /* Redefine Y strides to skip 2 lines at a time ... */
-  dest_stride *= 2;
-  src_y_stride *= 2;
-
-  for (ypos = 0; ypos < height / 2; ypos++) {
-
-    gst_alpha_chromakey_row_i420 (alpha, dest1, dest2,
-        srcY1, srcY2, srcU, srcV, width);
-
-    dest1 += dest_stride;
-    dest2 += dest_stride;
-    srcY1 += src_y_stride;
-    srcY2 += src_y_stride;
-    srcU += src_uv_stride;
-    srcV += src_uv_stride;
+    dest1 += wrap3;
+    dest2 += wrap3;
+    srcY1 += wrap;
+    srcY2 += wrap;
+    srcU += wrap2;
+    srcV += wrap2;
   }
 }
 
@@ -855,72 +787,109 @@ gst_alpha_init_params (GstAlpha * alpha)
   alpha->kg = MIN (kgl, 127);
 }
 
-static gboolean
-gst_alpha_start (GstBaseTransform * btrans)
-{
-  GstAlpha *alpha = GST_ALPHA (btrans);
-
-  gst_alpha_init_params (alpha);
-
-  return TRUE;
-}
-
 static GstFlowReturn
-gst_alpha_transform (GstBaseTransform * btrans, GstBuffer * in, GstBuffer * out)
+gst_alpha_chain (GstPad * pad, GstBuffer * buffer)
 {
-  GstAlpha *alpha = GST_ALPHA (btrans);
-  gint width, height;
-  GstClockTime timestamp;
+  GstAlpha *alpha;
+  GstBuffer *outbuf;
+  gint new_width, new_height;
+  GstFlowReturn ret;
 
-  width = alpha->width;
-  height = alpha->height;
+  alpha = GST_ALPHA (GST_PAD_PARENT (pad));
 
-  GST_BUFFER_TIMESTAMP (out) = GST_BUFFER_TIMESTAMP (in);
-  GST_BUFFER_DURATION (out) = GST_BUFFER_DURATION (in);
-  timestamp = gst_segment_to_stream_time (&btrans->segment, GST_FORMAT_TIME,
-      GST_BUFFER_TIMESTAMP (in));
-  GST_LOG ("Got stream time of %" GST_TIME_FORMAT, GST_TIME_ARGS (timestamp));
-  if (GST_CLOCK_TIME_IS_VALID (timestamp))
-    gst_object_sync_values (G_OBJECT (alpha), timestamp);
+  new_width = alpha->in_width;
+  new_height = alpha->in_height;
+
+  if (new_width != alpha->out_width ||
+      new_height != alpha->out_height || !GST_PAD_CAPS (alpha->srcpad)) {
+    GstCaps *newcaps;
+
+    newcaps = gst_caps_copy (gst_pad_get_negotiated_caps (alpha->sinkpad));
+    gst_caps_set_simple (newcaps,
+        "format", GST_TYPE_FOURCC, GST_STR_FOURCC ("AYUV"),
+        "width", G_TYPE_INT, new_width, "height", G_TYPE_INT, new_height, NULL);
+
+    gst_pad_set_caps (alpha->srcpad, newcaps);
+
+    alpha->out_width = new_width;
+    alpha->out_height = new_height;
+  }
+
+  outbuf =
+      gst_buffer_new_and_alloc (ROUND_UP_2 (new_width) *
+      ROUND_UP_2 (new_height) * 4);
+  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (alpha->srcpad));
+  GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (buffer);
+  GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (buffer);
 
   switch (alpha->method) {
     case ALPHA_METHOD_SET:
       if (alpha->ayuv) {
-        gst_alpha_set_ayuv (GST_BUFFER_DATA (in),
-            GST_BUFFER_DATA (out), width, height, alpha->alpha);
+        gst_alpha_set_ayuv (GST_BUFFER_DATA (buffer),
+            GST_BUFFER_DATA (outbuf), new_width, new_height, alpha->alpha);
       } else {
-        gst_alpha_set_i420 (GST_BUFFER_DATA (in),
-            GST_BUFFER_DATA (out), width, height, alpha->alpha);
+        gst_alpha_set_i420 (GST_BUFFER_DATA (buffer),
+            GST_BUFFER_DATA (outbuf), new_width, new_height, alpha->alpha);
       }
       break;
     case ALPHA_METHOD_GREEN:
     case ALPHA_METHOD_BLUE:
     case ALPHA_METHOD_CUSTOM:
       if (alpha->ayuv) {
-        gst_alpha_chroma_key_ayuv (GST_BUFFER_DATA (in),
-            GST_BUFFER_DATA (out), width, height, alpha);
+        gst_alpha_chroma_key_ayuv (GST_BUFFER_DATA (buffer),
+            GST_BUFFER_DATA (outbuf), new_width, new_height, alpha);
       } else {
-        gst_alpha_chroma_key_i420 (GST_BUFFER_DATA (in),
-            GST_BUFFER_DATA (out), width, height, alpha);
+        gst_alpha_chroma_key_i420 (GST_BUFFER_DATA (buffer),
+            GST_BUFFER_DATA (outbuf), new_width, new_height, alpha);
       }
       break;
     default:
       break;
   }
 
-  return GST_FLOW_OK;
+  gst_buffer_unref (buffer);
+
+  ret = gst_pad_push (alpha->srcpad, outbuf);
+
+  return ret;
+}
+
+static GstStateChangeReturn
+gst_alpha_change_state (GstElement * element, GstStateChange transition)
+{
+  GstAlpha *alpha;
+
+  alpha = GST_ALPHA (element);
+
+  switch (transition) {
+    case GST_STATE_CHANGE_NULL_TO_READY:
+      break;
+    case GST_STATE_CHANGE_READY_TO_PAUSED:
+      gst_alpha_init_params (alpha);
+      break;
+    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
+      break;
+    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+      break;
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      break;
+    case GST_STATE_CHANGE_READY_TO_NULL:
+      break;
+  }
+
+  parent_class->change_state (element, transition);
+
+  return GST_STATE_CHANGE_SUCCESS;
 }
 
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  gst_controller_init (NULL, NULL);
-
   return gst_element_register (plugin, "alpha", GST_RANK_NONE, GST_TYPE_ALPHA);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "alpha",
-    "adds an alpha channel to video - constant or via chroma-keying",
+    "adds an alpha channel to video",
     plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
