@@ -1,7 +1,6 @@
 /* GStreamer
  * Copyright (C) <2005> Philippe Khalaf <burger@speedy.org>
  * Copyright (C) <2005> Nokia Corporation <kai.vehmanen@nokia.com>
- * Copyright (C) <2006> Joni Valtanen <joni.valtanen@movial.fi>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,21 +23,9 @@
 #endif
 #include "gstudp-marshal.h"
 #include "gstdynudpsink.h"
-
-#include <stdio.h>
-#include <stdlib.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <errno.h>
-#include <string.h>
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-#include <sys/types.h>
 #include <gst/netbuffer/gstnetbuffer.h>
 
-GST_DEBUG_CATEGORY_STATIC (dynudpsink_debug);
+GST_DEBUG_CATEGORY (dynudpsink_debug);
 #define GST_CAT_DEFAULT (dynudpsink_debug)
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -47,7 +34,7 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS_ANY);
 
 /* elementfactory information */
-static const GstElementDetails gst_dynudpsink_details =
+static GstElementDetails gst_dynudpsink_details =
 GST_ELEMENT_DETAILS ("UDP packet sender",
     "Sink/Network",
     "Send data over the network via UDP",
@@ -65,31 +52,20 @@ enum
   LAST_SIGNAL
 };
 
-#define UDP_DEFAULT_SOCKFD		-1
-#define UDP_DEFAULT_CLOSEFD		TRUE
-
 enum
 {
   PROP_0,
-  PROP_SOCKFD,
-  PROP_CLOSEFD
+  PROP_SOCKFD
+      /* FILL ME */
 };
 
-#define CLOSE_IF_REQUESTED(udpctx)                                        \
-G_STMT_START {                                                            \
-  if ((!udpctx->externalfd) || (udpctx->externalfd && udpctx->closefd)) { \
-    CLOSE_SOCKET(udpctx->sock);                                           \
-    if (udpctx->sock == udpctx->sockfd)                                   \
-      udpctx->sockfd = UDP_DEFAULT_SOCKFD;                                \
-  }                                                                       \
-  udpctx->sock = -1;                                                      \
-} G_STMT_END
-
 static void gst_dynudpsink_base_init (gpointer g_class);
 static void gst_dynudpsink_class_init (GstDynUDPSink * klass);
 static void gst_dynudpsink_init (GstDynUDPSink * udpsink);
 static void gst_dynudpsink_finalize (GObject * object);
 
+static void gst_dynudpsink_get_times (GstBaseSink * sink, GstBuffer * buffer,
+    GstClockTime * start, GstClockTime * end);
 static GstFlowReturn gst_dynudpsink_render (GstBaseSink * sink,
     GstBuffer * buffer);
 static void gst_dynudpsink_close (GstDynUDPSink * sink);
@@ -153,7 +129,7 @@ gst_dynudpsink_class_init (GstDynUDPSink * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasesink_class = (GstBaseSinkClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
+  parent_class = g_type_class_ref (GST_TYPE_BASE_SINK);
 
   gobject_class->set_property = gst_dynudpsink_set_property;
   gobject_class->get_property = gst_dynudpsink_get_property;
@@ -167,28 +143,24 @@ gst_dynudpsink_class_init (GstDynUDPSink * klass)
 
   g_object_class_install_property (gobject_class, PROP_SOCKFD,
       g_param_spec_int ("sockfd", "socket handle",
-          "Socket to use for UDP sending. (-1 == allocate)",
-          -1, G_MAXINT16, UDP_DEFAULT_SOCKFD, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class, PROP_CLOSEFD,
-      g_param_spec_boolean ("closefd", "Close sockfd",
-          "Close sockfd if passed as property on state change",
-          UDP_DEFAULT_CLOSEFD, G_PARAM_READWRITE));
+          "Socket to use for UDP reception.",
+          0, G_MAXINT16, 0, G_PARAM_READWRITE));
 
   gstelement_class->change_state = gst_dynudpsink_change_state;
 
+  gstbasesink_class->get_times = gst_dynudpsink_get_times;
   gstbasesink_class->render = gst_dynudpsink_render;
 
   GST_DEBUG_CATEGORY_INIT (dynudpsink_debug, "dynudpsink", 0, "UDP sink");
 }
 
+
 static void
 gst_dynudpsink_init (GstDynUDPSink * sink)
 {
-  WSA_STARTUP (sink);
+  GstDynUDPSink *udpsink;
 
-  sink->sockfd = UDP_DEFAULT_SOCKFD;
-  sink->closefd = UDP_DEFAULT_CLOSEFD;
-  sink->externalfd = FALSE;
+  udpsink = GST_DYNUDPSINK (sink);
 
   sink->sock = -1;
 }
@@ -196,16 +168,15 @@ gst_dynudpsink_init (GstDynUDPSink * sink)
 static void
 gst_dynudpsink_finalize (GObject * object)
 {
-  GstDynUDPSink *udpsink;
-
-  udpsink = GST_DYNUDPSINK (object);
-
-  if (udpsink->sockfd >= 0 && udpsink->closefd)
-    CLOSE_SOCKET (udpsink->sockfd);
-
   G_OBJECT_CLASS (parent_class)->finalize (object);
+}
 
-  WSA_CLEANUP (object);
+static void
+gst_dynudpsink_get_times (GstBaseSink * sink, GstBuffer * buffer,
+    GstClockTime * start, GstClockTime * end)
+{
+  *start = GST_BUFFER_TIMESTAMP (buffer);
+  *end = *start + GST_BUFFER_DURATION (buffer);
 }
 
 static GstFlowReturn
@@ -216,8 +187,6 @@ gst_dynudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
   guint8 *data;
   GstNetBuffer *netbuf;
   struct sockaddr_in theiraddr;
-  guint16 destport;
-  guint32 destaddr;
 
   memset (&theiraddr, 0, sizeof (theiraddr));
 
@@ -234,6 +203,8 @@ gst_dynudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
   data = GST_BUFFER_DATA (netbuf);
 
   GST_DEBUG ("about to send %d bytes", size);
+  guint16 destport;
+  guint32 destaddr;
 
   // let's get the address from the netbuffer
   gst_netaddress_get_ip4_address (&netbuf->to, &destaddr, &destport);
@@ -243,11 +214,7 @@ gst_dynudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
   theiraddr.sin_family = AF_INET;
   theiraddr.sin_addr.s_addr = destaddr;
   theiraddr.sin_port = destport;
-#ifdef G_OS_WIN32
-  ret = sendto (sink->sock, (char *) data, size, 0,
-#else
   ret = sendto (sink->sock, data, size, 0,
-#endif
       (struct sockaddr *) &theiraddr, sizeof (theiraddr));
 
   if (ret < 0) {
@@ -277,14 +244,8 @@ gst_dynudpsink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_SOCKFD:
-      if (udpsink->sockfd >= 0 && udpsink->sockfd != udpsink->sock &&
-          udpsink->closefd)
-        CLOSE_SOCKET (udpsink->sockfd);
-      udpsink->sockfd = g_value_get_int (value);
-      GST_DEBUG ("setting SOCKFD to %d", udpsink->sockfd);
-      break;
-    case PROP_CLOSEFD:
-      udpsink->closefd = g_value_get_boolean (value);
+      udpsink->sock = g_value_get_int (value);
+      GST_DEBUG ("setting SOCKFD to %d", udpsink->sock);
       break;
 
     default:
@@ -303,11 +264,9 @@ gst_dynudpsink_get_property (GObject * object, guint prop_id, GValue * value,
 
   switch (prop_id) {
     case PROP_SOCKFD:
-      g_value_set_int (value, udpsink->sockfd);
-      break;
-    case PROP_CLOSEFD:
-      g_value_set_boolean (value, udpsink->closefd);
+      g_value_set_int (value, udpsink->sock);
       break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -322,9 +281,10 @@ gst_dynudpsink_init_send (GstDynUDPSink * sink)
   guint bc_val;
   gint ret;
 
-  if (sink->sockfd == -1) {
+  if (sink->sock == -1) {
     /* create sender socket if none available */
-    if ((sink->sock = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
+
+    if ((sink->sock = socket (AF_INET, SOCK_DGRAM, 0)) == -1)
       goto no_socket;
 
     bc_val = 1;
@@ -332,12 +292,8 @@ gst_dynudpsink_init_send (GstDynUDPSink * sink)
             setsockopt (sink->sock, SOL_SOCKET, SO_BROADCAST, &bc_val,
                 sizeof (bc_val))) < 0)
       goto no_broadcast;
-
-    sink->externalfd = TRUE;
-  } else {
-    sink->sock = sink->sockfd;
-    sink->externalfd = TRUE;
   }
+
   return TRUE;
 
   /* ERRORS */
@@ -349,7 +305,6 @@ no_socket:
 no_broadcast:
   {
     perror ("setsockopt");
-    CLOSE_IF_REQUESTED (sink);
     return FALSE;
   }
 }
@@ -363,7 +318,7 @@ gst_dynudpsink_get_stats (GstDynUDPSink * sink, const gchar * host, gint port)
 static void
 gst_dynudpsink_close (GstDynUDPSink * sink)
 {
-  CLOSE_IF_REQUESTED (sink);
+  close (sink->sock);
 }
 
 static GstStateChangeReturn
