@@ -1,9 +1,6 @@
-/*
- * GStreamer - SunAudio sink
+/* GStreamer
  * Copyright (C) 2004 David A. Schleef <ds@schleef.org>
- * Copyright (C) 2005,2006 Sun Microsystems, Inc.,
- *               Brian Cameron <brian.cameron@sun.com>
- * Copyright (C) 2006 Jan Schmidt <thaytan@mad.scientist.com>
+ * Copyright (C) 2005 Brian Cameron <brian.cameron@sun.com>
  *
  * gstsunaudiosink.h:
  *
@@ -32,11 +29,11 @@
 
 G_BEGIN_DECLS
 
-#define GST_TYPE_SUNAUDIO_SINK            (gst_sunaudiosink_get_type())
-#define GST_SUNAUDIO_SINK(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_SUNAUDIO_SINK,GstSunAudioSink))
-#define GST_SUNAUDIO_SINK_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_SUNAUDIO_SINK,GstSunAudioSinkClass))
-#define GST_IS_SUNAUDIO_SINK(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_SUNAUDIO_SINK))
-#define GST_IS_SUNAUDIO_SINK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_SUNAUDIO_SINK))
+#define GST_TYPE_SUNAUDIO_SINK     (gst_sunaudiosink_get_type())
+#define GST_SUNAUDIO_SINK(obj)     (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_SUNAUDIO_SINK,GstSunAudioSink))
+#define GST_SUNAUDIO_SINK_CLASS(klass)  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_SUNAUDIO_SINK,GstSunAudioSinkClass))
+#define GST_IS_SUNAUDIO_SINK(obj)       (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_SUNAUDIO_SINK))
+#define GST_IS_SUNAUDIO_SINK_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_SUNAUDIO_SINK))
 
 typedef struct _GstSunAudioSink GstSunAudioSink;
 typedef struct _GstSunAudioSinkClass GstSunAudioSinkClass;
@@ -50,20 +47,8 @@ struct _GstSunAudioSink {
   audio_device_t dev;
   audio_info_t info;
 
-  /* Number of segments the ringbuffer is configured for */
-  guint segtotal;
-  guint segtotal_samples;
-
-  /* Number of segments written to the device */
-  gint segs_written;
-  /* Number of samples written to the device */
-  gint samples_written;
-  guint bytes_per_sample;
-
-  /* mutex and gcond used to control the write method */
-  GMutex *write_mutex;
-  GCond *sleep_cond;
-  gboolean flushing;
+  gint   bytes_per_sample;
+  gint   buffer_size;
 };
 
 struct _GstSunAudioSinkClass {
